<?php

/**
 * @file
 * This file includes helper functions for creating reports for dquarks.module
 */

// All functions within this file need the dquarks_submissions.inc.
module_load_include('inc', 'dquarks', 'includes/dquarks_submissions');

/**
 * Retrieve lists of submissions for a given dquarks.
 */
function dquarks_results_submissions($node, $user_filter, $pager_count) {
  global $user;

  if (isset($_GET['results']) && is_numeric($_GET['results'])) {
    $pager_count = $_GET['results'];
  }

  $header = theme('dquarks_results_submissions_header', $node);
  if ($user_filter) {
    drupal_set_title(t('Submissions for %user', array('%user' => $user->name)));
    $submissions = dquarks_get_submissions($node->nid, $header, $user->uid, $pager_count);
    $count = dquarks_get_submission_count($node->nid, $user->uid);
  }
  else {
    $submissions = dquarks_get_submissions($node->nid, $header, NULL, $pager_count);
    $count = dquarks_get_submission_count($node->nid);
  }
  return theme('dquarks_results_submissions', $node, $submissions, $count, $pager_count);
}

/**
 * Theme the list of links for selecting the number of results per page.
 *
 * @param int $total_count
 *   The total number of results available.
 * @param int $pager_count
 *   The current number of results displayed per page.
 */
function theme_dquarks_results_per_page($total_count, $pager_count) {
  $output = '';

  // Create a list of results-per-page options.
  $counts = array(
    '20' => '20',
    '50' => '50',
    '100' => '100',
    '200' => '200',
    '500' => '500',
    '1000' => '1000',
    '0' => t('All'),
  );

  $count_links = array();

  foreach ($counts as $number => $text) {
    if ($number < $total_count) {
      $count_links[] = l($text, $_GET['q'], array('query' => 'results=' . $number, 'attributes' => array('class' => $pager_count == $number ? 'selected' : '')));
    }
  }

  $output .= '<div class="dquarks-results-per-page">';
  if (count($count_links) > 1) {
    $output .= t('Show !count results per page.', array('!count' => implode(' | ', $count_links)));
  }
  else {
    $output .= t('Showing all results.');
  }
  if ($total_count > 1) {
    $output .= ' ' . t('@total results total.', array('@total' => $total_count));
  }
  $output .= '</div>';

  return $output;
}

/**
 * Theme the header of the submissions table.
 *
 * This is done in it's own function so that dquarks can retrieve the header and
 * use it for sorting the results.
 */
function theme_dquarks_results_submissions_header($node) {
  $columns = array(
    array('data' => t('#'), 'field' => 'sid', 'sort' => 'asc'),
    array('data' => t('Submitted'), 'field' => 'submitted'),
  );
  if (user_access('access dquarks results')) {
    $columns[] = array('data' => t('User'), 'field' => 'name');
    $columns[] = array('data' => t('IP Address'), 'field' => 'remote_addr');
  }
  $columns[] = array('data' => t('Operations'), 'colspan' => 3);

  return $columns;
}

/**
 * Theme the submissions tab of the dquarks results page.
 *
 * @param object $node
 *   The node whose results are being displayed.
 * @param object $submissions
 *   An array of all submissions for this dquarks.
 * @param int $total_count
 *   The total number of submissions to this dquarks.
 * @param int $pager_count
 *   The number of results to be shown per page.
 */
function theme_dquarks_results_submissions($node, $submissions, $total_count = 0, $pager_count = 0) {
  global $user;

  drupal_add_css(drupal_get_path('module', 'dquarks') . '/css/dquarks.css');

  // This header has to be generated separately so we can add the SQL necessary
  // to sort the results.
  $header = theme('dquarks_results_submissions_header', $node);

  $rows = array();
  foreach ($submissions as $sid => $submission) {
    $draftstring = "";
    if ($submission->is_draft) {
      $draftstring = " (draft)";
    }
    $row = array(
      $sid . $draftstring,
      format_date($submission->submitted, 'small'),
    );
    if (user_access('access dquarks results')) {
      $row[] = theme('username', $submission);
      $row[] = $submission->remote_addr;
    }
    $row[] = l(t('View'), "node/$node->nid/quizsubmission/$sid");
    if ((user_access('edit own quiz submissions') && ($user->uid == $submission->uid)) || user_access('edit quiz submissions')) {
      $row[] = l(t('Edit'), "node/$node->nid/quizsubmission/$sid/edit");
      $row[] = l(t('Delete'), "node/$node->nid/quizsubmission/$sid/delete", array('query' => drupal_get_destination()));
    }
    else {
      $row[count($row) - 1] = array('data' => $row[count($row) - 1], 'colspan' => 3);
    }
    $rows[] = $row;
  }

  if (count($rows) == 0) {
    $rows[] = array(array(
        'data' => t('There are no submissions for this quiz. <a href=\"!url\">View this quiz</a>.', array(
          '!url' => url('node/' . $node->nid))),
        'colspan' => 7),
    );
  }

  $output = '';
  $output .= theme('dquarks_results_per_page', $total_count, $pager_count);
  $output .= theme('table', $header, $rows);
  if (arg(2) == 'submissions') {
    $output .= theme('links', array('dquarks' => array('title' => t('Go back to the quiz'), 'href' => 'node/' . $node->nid)));
  }
  if ($pager_count) {
    $output .= theme('pager', NULL, $pager_count, 0);
  }
  return $output;
}

/**
 * Create a table containing all submitted values for a dquarks node.
 */
function dquarks_results_table($node, $pager_count = 0) {
  // Load Components.
  dquarks_load_components();

  if (isset($_GET['results']) && is_numeric($_GET['results'])) {
    $pager_count = $_GET['results'];
  }

  // Get all the submissions for the node.
  $header = theme('dquarks_results_table_header', $node);
  $submissions = dquarks_get_submissions($node->nid, $header, NULL, $pager_count);
  $total_count = dquarks_get_submission_count($node->nid);

  $output = theme('dquarks_results_table', $node, $node->dquarks['components'], $submissions, $total_count, $pager_count);
  if ($pager_count) {
    $output .= theme('pager', NULL, $pager_count, 0);
  }
  return $output;
}

/**
 * Format the output.
 *
 * @param object $node
 *   The node whose results are being displayed.
 *
 * @return array
 *   The fields to display
 */
function theme_dquarks_results_table_header($node) {
  return array(
    array('data' => t('#'), 'field' => 'sid', 'sort' => 'asc'),
    array('data' => t('Submitted'), 'field' => 'submitted'),
    array('data' => t('User'), 'field' => 'name'),
    array('data' => t('IP Address'), 'field' => 'remote_addr'),
    array('data' => t('Score'), 'field' => 'score'),
  );
}

/**
 * Theme the results table displaying all the submissions for a particular node.
 *
 * @param object $node
 *   The node whose results are being displayed.
 * @param array $components
 *   An associative array of the components for this dquarks.
 * @param object $submissions
 *   An array of all submissions for this dquarks.
 * @param int $total_count
 *   The total number of submissions to this dquarks.
 * @param int $pager_count
 *   The number of results to be shown per page.
 */
function theme_dquarks_results_table($node, $components, $submissions, $total_count, $pager_count) {
  drupal_add_css(drupal_get_path('module', 'dquarks') . '/css/dquarks.css');

  $header = array();
  $rows = array();
  $cell = array();

  // This header has to be generated seperately so we can add the SQL necessary.
  // to sort the results.
  $header = theme('dquarks_results_table_header', $node);

  // Generate a row for each submission.
  foreach ($submissions as $sid => $submission) {
    $cell[] = l($sid, 'node/' . $node->nid . '/quizsubmission/' . $sid);
    $cell[] = format_date($submission->submitted, 'small');
    $cell[] = theme('username', $submission);
    $cell[] = $submission->remote_addr;
    $cell[] = $submission->score . '/' . $submission->max_score;
    $component_headers = array();

    // Generate a cell for each component.
    foreach ($node->dquarks['components'] as $component) {
      $table_function = '_dquarks_table_data_' . $component['type'];
      if (function_exists($table_function)) {
        $data = isset($submission->data[$component['cid']]) ? $submission->data[$component['cid']] : NULL;
        $submission_output = $table_function($data, $component);
        if ($submission_output !== NULL) {
          $component_headers[] = $component['name'];
          $cell[] = $submission_output;
        }
      }
    }

    $rows[] = $cell;
    unset($cell);
  }
  if (!empty($component_headers)) {
    $header = array_merge($header, $component_headers);
  }

  if (count($rows) == 0) {
    $rows[] = array(array(
        'data' => t('There are no submissions for this quiz. <a href=\"!url\">View this quiz</a>.', array(
          '!url' => url('node/' . $node->nid))),
        'colspan' => 5),
    );
  }


  $output = '';
  $output .= theme('dquarks_results_per_page', $total_count, $pager_count);
  $output .= theme('table', $header, $rows);
  return $output;
}

/**
 * Delete all submissions for a node.
 *
 * @param int $nid
 *   The node id whose submissions will be deleted.
 */
function dquarks_results_clear($nid) {
  $node = node_load($nid);
  $submissions = dquarks_get_submissions($nid);
  foreach ($submissions as $submission) {
    dquarks_submission_delete($node, $submission);
  }
}

/**
 * Confirmation form to delete all submissions for a node.
 *
 * @param object $node
 *   ID of node whose submissions will be cleared.
 */
function dquarks_results_clear_form($form_state, $node) {
  drupal_set_title(t('Clear quiz Submissions'));

  $form = array();
  $form['nid'] = array('#type' => 'value', '#value' => $node->nid);
  $question = t('Are you sure you want to delete all submissions for this quiz?');

  return confirm_form($form, $question, 'node/' . $node->nid . '/dquarks-results', NULL, t('Clear'), t('Cancel'));
}

/**
 * Clear results.
 *
 * @param array $form
 *   An array of form items of submission.
 * @param array $form_state
 *   An array of form items of submission.
 */
function dquarks_results_clear_form_submit($form, &$form_state) {
  dquarks_results_clear($form_state['values']['nid']);
  $node = node_load(array('nid' => $form_state['values']['nid']));
  $title = $node->title;

  $message = t('%title quiz entries cleared.', array('%title' => $title));
  drupal_set_message($message);
  watchdog('dquarks', $message);
  $form_state['redirect'] = 'node/' . $form_state['values']['nid'] . '/dquarks-results';
}

/**
 * Form to configure the download of excel files.
 */
function dquarks_results_download_form(&$form_state, $node) {
  module_load_include('inc', 'dquarks', 'includes/dquarks_export');

  $form = array();

  $form['node'] = array(
    '#type' => 'value',
    '#value' => $node,
  );

  $form['format'] = array(
    '#type' => 'radios',
    '#title' => t('Export format'),
    '#options' => dquarks_export_list(),
    '#default_value' => 'excel',
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Download'),
  );

  return $form;
}

/**
 * Validate handler for dquarks_results_download_form().
 */
function dquarks_results_download_form_submit(&$form, &$form_state) {
  dquarks_results_download($form_state['values']['node'], $form_state['values']['format'], $form_state['values']);
}

/**
 * Function used to sanitize node's title before sending them to PHPExcel.
 *
 * @param object $node
 *   The dquarks node object.
 */
function _dquarks_sanitize_node(&$node) {
  $forbidden_char = array(
    '*', ':', '/', '\\', '?', '[', ']',
  );

  $node->title = str_replace($forbidden_char, '', $node->title);
}

/**
 * Generate a Excel xlsx file containing all submissions for a dquarks.
 */
function dquarks_results_download($node, $format = 'delimiter', $options = array()) {
  module_load_include('inc', 'dquarks', 'includes/dquarks_export');

  if (empty($options)) {
    $options = array(
      'delimiter' => variable_get('dquarks_csv_delimiter', '\t'),
      'format' => variable_get('dquarks_export_format', 'delimited'),
    );
  }

  // Open a new dquarks exporter object.
  $exporter = dquarks_export_create_handler($format, $options);
  $file_name = tempnam(variable_get('file_directory_temp', file_directory_temp()), 'dquarks');

  _dquarks_sanitize_node($node);
  $exporter->setContent($file_name, $node, $format, $options);

  $export_name = _dquarks_safe_name($node->title);
  $exporter->setHeaders($export_name);
  // The @ makes it silent.
  @readfile($file_name);
  // Clean up, the @ makes it silent.
  @unlink($file_name);
  exit();
}

/**
 * Provides a simple analysis of all submissions to a dquarks.
 *
 * @param object $node
 *   The dquarks node on which to generate the analysis.
 * @param array $sids
 *   An array of submission IDs to which this analysis may be filtered. May be
 *   used to generate results that are per-user or other groups of submissions.
 */
function dquarks_results_analysis($node, $sids = array()) {
  $rows = array();
  $question_number = 0;

  $headers = array(
    t('Q'),
    array('data' => t('responses'), 'colspan' => '10'),
  );

  // Load all component types.
  dquarks_load_components();
  foreach ($node->dquarks['components'] as $component) {

    // Do component specific call.
    $analysis_function = '_dquarks_analysis_rows_' . $component['type'];
    if (function_exists($analysis_function)) {
      $question_number++;
      $crows = $analysis_function($component, $sids);
      if (is_array($crows)) {
        $row[0] = array(
          'data' => '<strong>' . $question_number . '</strong>',
          'rowspan' => count($crows) + 1,
          'valign' => 'top',
        );
        $row[1] = array(
          'data' => '<strong>' . $component['name'] . '</strong>',
          'colspan' => '10',
        );
        $rows = array_merge($rows, array_merge(array($row), $crows));
      }
    }
  }

  if (count($rows) == 0) {
    $rows[] = array(array(
        'data' => t('There are no submissions for this quiz. <a href=\"!url\">View this quiz</a>.', array(
          '!url' => url('node/' . $node->nid))),
        'colspan' => 20),
    );
  }

  return theme('table', $headers, $rows);
}
