<?php

/**
 * @file
 * This file is loaded when handling submissions, either submitting new,
 * editing, or viewing. It also contains all CRUD functions for submissions.
 */

/**
 * Update a dquarks submission entry in the database.
 *
 * @param object $node
 *   The node object containing the current dquarks.
 * @param int $sid
 *   The id of submission.
 * @param array $submitted
 *   The already submitted submisson.
 * @param boolean $is_draft
 *   True if the submission is draft.
 *
 * @return int
 *   The existing submission SID.
 */
function dquarks_submission_update($node, $sid, $submitted, $is_draft = 0) {
  global $user;
  // Update the submission data by first removing all this submissions data.
  db_query("DELETE FROM {dquarks_submissions} WHERE sid = %d", $sid);
  db_query("INSERT INTO {dquarks_submissions} (nid, sid, uid, submitted, remote_addr, is_draft) " . " VALUES (%d, %d, %d, %d, '%s', %d)", $node->nid, $sid, $user->uid, time(), ip_address(), $is_draft);

  // If is draft, only delete data for components submitted, to
  // preserve any data from form pages not visited in this submission.
  if ($is_draft) {
    $_submitted_cids = array();
    foreach ($submitted as $cid => $value) {
      $_submitted_cids[] = $cid;
    }
    db_query("DELETE FROM {dquarks_submitted_data} WHERE sid = %d AND cid IN (" . implode(', ', $_submitted_cids) . ")", $sid);
  }
  else {
    db_query("DELETE FROM {dquarks_submitted_data} WHERE sid = %d", $sid);
  }
  // Then re-add it to the database.
  foreach ($submitted as $cid => $value) {
    // Don't save pagebreaks and directions as submitted data.
    if ($node->dquarks['components'][$cid]['type'] == 'pagebreak' || $node->dquarks['components'][$cid]['type'] == 'direction') {
      continue;
    }

    if (is_array($value)) {
      $delta = 0;
      foreach ($value as $k => $v) {
        db_query("INSERT INTO {dquarks_submitted_data} (nid, sid, cid, no, data) VALUES (%d, %d, %d, %d, '%s')", $node->nid, $sid, $cid, $delta, $v);
        $delta++;
      }
    }
    else {
      db_query("INSERT INTO {dquarks_submitted_data} (nid, sid, cid, no, data) VALUES (%d, %d, %d, %d, '%s')", $node->nid, $sid, $cid, 0, $value);
    }
  }

  return $sid;
}

/**
 * Insert a dquarks submission entry in the database.
 *
 * @param object $node
 *   The node object containing the current dquarks.
 * @param object $submitted
 *   The dquarks submission object to be saved into the database.
 * @param boolean $is_draft
 *   The quizz is draft or published
 *
 * @return int
 *   The new submission SID.
 */
function dquarks_submission_insert($node, $submitted, $is_draft = 0) {
  global $user;

  $result = db_query("INSERT INTO {dquarks_submissions} (nid, uid, submitted, remote_addr, is_draft) " . " VALUES (%d, %d, %d, '%s', %d)", $node->nid, $user->uid, time(), ip_address(), $is_draft);

  $sid = db_last_insert_id('dquarks_submissions', 'sid');

  foreach ($submitted as $cid => $value) {
    // Don't save pagebreaks and directions as submitted data.
    if ($node->dquarks['components'][$cid]['type'] == 'pagebreak' || $node->dquarks['components'][$cid]['type'] == 'direction') {
      continue;
    }

    if (is_array($value)) {
      $delta = 0;
      foreach ($value as $k => $v) {
        db_query("INSERT INTO {dquarks_submitted_data} (nid, sid, cid, no, data) VALUES (%d, %d, %d, %d, '%s')", $node->nid, $sid, $cid, $delta, $v);
        $delta++;
      }
    }
    else {
      db_query("INSERT INTO {dquarks_submitted_data} (nid, sid, cid, no, data) VALUES (%d, %d, %d, %d, '%s')", $node->nid, $sid, $cid, 0, $value);
    }
  }

  return $sid;
}

/**
 * Delete a single submission.
 *
 * @param object $node
 *   The node for which this dquarks was submitted.
 * @param object $submission
 *   The submission to be deleted (from dquarks_submitted_data).
 */
function dquarks_submission_delete($node, $submission) {
  // Iterate through all components and let each do cleanup if necessary.
  dquarks_load_components();
  foreach ($node->dquarks['components'] as $cid => $component) {
    $delete_function = '_dquarks_delete_' . $component['type'];
    if (function_exists($delete_function) && isset($submission->data[$cid])) {
      $delete_function($submission->data[$cid], $component);
    }
  }

  db_query('DELETE FROM {dquarks_submitted_data} WHERE nid = %d AND sid = %d', $node->nid, $submission->sid);
  db_query('DELETE FROM {dquarks_submissions} WHERE nid = %d AND sid = %d', $node->nid, $submission->sid);
}

/**
 * Confirm form to delete a single form submission.
 *
 * @param array $form_state
 *   The current form state.
 * @param object $node
 *   The node for which this dquarks was submitted.
 * @param array $submission
 *   The submission to be deleted (from dquarks_submitted_data).
 */
function dquarks_submission_delete_form($form_state, $node, $submission) {
  drupal_set_title(t('Delete Quiz Submission'));

  $form = array();
  $form['node'] = array('#type' => 'value', '#value' => $node);
  $form['submission'] = array('#type' => 'value', '#value' => $submission);
  $question = t('Are you sure you want to delete this submission?');
  if (isset($_GET['destination'])) {
    $destination = $_GET['destination'];
  }
  elseif (user_access('access dquarks results')) {
    $destination = 'node/' . $node->nid . '/dquarks-results';
  }
  else {
    $destination = 'node/' . $node->nid . '/quizsubmissions';
  }

  return confirm_form($form, NULL, check_plain($destination), check_plain($question), t('Delete'), t('Cancel'));
}

/**
 * Delete selected submission.
 *
 * @param array $form
 *   An array of form items.
 * @param array $form_state
 *   An array of form values items.
 */
function dquarks_submission_delete_form_submit($form, &$form_state) {
  dquarks_submission_delete($form_state['values']['node'], $form_state['values']['submission']);
  drupal_set_message(t('Submission deleted.'));

  $form_state['redirect'] = 'node/' . $form_state['values']['node']->nid . '/dquarks-results';
}

/**
 * Return all the submissions for a particular node.
 *
 * @param int $nid
 *   The node ID for which submissions are being fetched.
 * @param array $header
 *   If the results of this fetch will be used in a sortable
 *   table, pass the array header of the table.
 * @param int $uid
 *   Optional; the user ID to filter the submissions by.
 *
 * @return array
 *   An array of submissions matching your filters.
 */
function dquarks_get_submissions($nid, $header = NULL, $uid = NULL, $pager_count = 0) {
  $sids = NULL;
  if ($pager_count) {
    $pager_query = 'SELECT * FROM {dquarks_submissions} WHERE nid = %d';
    if ($uid) {
      $pager_query .= ' AND uid = %d';
    }
    $pager_query .= is_array($header) ? tablesort_sql($header) : ' ORDER BY sid ASC';

    $res = pager_query($pager_query, $pager_count, 0, NULL, $nid, $uid);
    $sids = array();
    while ($row = db_fetch_object($res)) {
      $sids[] = $row->sid;
    }
    $sids = implode($sids, ',');
  }

  $query = 'SELECT s.*, sd.cid, sd.no, sd.data, u.name, u.mail, u.status ';
  $query .= 'FROM {dquarks_submissions} s LEFT JOIN {dquarks_submitted_data} sd ';
  $query .= 'ON sd.sid = s.sid LEFT JOIN {users} u ON u.uid = s.uid ';
  $query .= 'WHERE sd.nid = %d';

  if ($pager_count && !empty($sids)) {
    $query .= ' AND s.sid IN (%s)';
  }

  if ($uid) {
    $query .= ' AND u.uid = %d';
  }

  if (is_array($header)) {
    $query .= tablesort_sql($header);
    if (!isset($header[0]['sort'])) {
      $query .= ', sid ASC';
    }
    $query .= ', cid ASC, no ASC';
  }
  else {
    $query .= ' ORDER BY sid ASC, cid ASC, no ASC';
  }

  $res = $pager_count ? db_query($query, $nid, $sids, $uid) : db_query($query, $nid, $uid);
  $submissions = array();
  $previous = array();

  // Outer loop: iterate for each submission.
  while ($row = db_fetch_object($res)) {
    if ($row->sid != $previous) {
      $submissions[$row->sid] = new stdClass();
      $submissions[$row->sid]->sid = $row->sid;
      $submissions[$row->sid]->submitted = $row->submitted;
      $submissions[$row->sid]->remote_addr = $row->remote_addr;
      $submissions[$row->sid]->uid = $row->uid;
      $submissions[$row->sid]->name = $row->name;
      $submissions[$row->sid]->status = $row->status;
      $submissions[$row->sid]->is_draft = $row->is_draft;
      $submissions[$row->sid]->score = $row->score;
      $submissions[$row->sid]->max_score = $row->max_score;
    }
    $submissions[$row->sid]->data[$row->cid]['value'][$row->no] = $row->data;
    $previous = $row->sid;
  }

  return $submissions;
}

/**
 * Return a count of the total number of submissions for a node.
 *
 * @param int $nid
 *   The node ID for which submissions are being fetched.
 * @param int $uid
 *   Optional; the user ID to filter the submissions by.
 *
 * @return int
 *   An integer value of the number of submissions.
 */
function dquarks_get_submission_count($nid, $uid = NULL) {
  $query = 'SELECT count(*) FROM {dquarks_submissions} WHERE nid = %d';
  if ($uid) {
    $query .= ' AND uid = %d';
  }
  return db_result(db_query($query, $nid, $uid));
}

/**
 * Fetch a specified submission for a dquarks node.
 */
function dquarks_get_submission($nid, $sid, $reset = FALSE) {
  static $submissions = array();

  if (!isset($submissions[$nid][$sid]) || $reset) {
    $query = 'SELECT s.*, sd.cid, sd.no, sd.data ' .
    'FROM {dquarks_submitted_data} as sd ' .
    'LEFT JOIN {dquarks_submissions} as s on (sd.sid = s.sid) ' .
    'WHERE sd.sid = %d AND s.nid = %d';

    $result = db_query($query, $sid, $nid);
    if ($row = db_fetch_object($result)) {
      $submission = new stdClass();
      $submission->nid = $row->nid;
      $submission->sid = $row->sid;
      $submission->uid = $row->uid;
      $submission->remote_addr = $row->remote_addr;
      $submission->submitted = $row->submitted;

      while ($row) {
        $submission->data[$row->cid]['value'][$row->no] = $row->data;
        $row = db_fetch_object($result);
      }
    }
    $submissions[$nid][$sid] = isset($submission) ? $submission : NULL;
  }

  return $submissions[$nid][$sid];
}

/**
 * Check if they are attempting to spam.
 *
 * @param string $to
 *   Destination e-mail.
 * @param string $subject
 *   Subject of e-mail.
 * @param array $from
 *   Source of e-mail.
 * @param array $headers
 *   Header of e-mail.
 *
 * @return boolean
 *   return false if not spam.
 */
function _dquarks_submission_spam_check($to, $subject, $from, $headers = array()) {
  $headers = implode('\n', (array) $headers);
  // Check if they are attempting to spam using a bcc or content type hack.
  if (preg_match('/(b?cc\s?:)|(content\-type:)/i', $to . "\n" . $subject . "\n" . $from . "\n" . $headers)) {
    // Possible spam attempt.
    return TRUE;
  }
  // Not spam.
  return FALSE;
}

/**
 * Check if the current user has exceeded the limit on this form.
 *
 * @param object $node
 *   The dquarks node to be checked.
 *
 * @return boolean
 *   Boolean TRUE if the user has exceeded their limit. FALSE otherwise.
 */
function _dquarks_submission_limit_check($node) {
  global $user;

  // Check if submission limiting is enabled.
  if ($node->dquarks['submit_limit'] == '-1') {
    // No check enabled.
    return FALSE;
  }

  // Retrieve submission data for this IP address or username from the database.
  $query = 'SELECT count(*) FROM {dquarks_submissions} ' .
  "WHERE (( 0 = %d AND remote_addr = '%s') OR (uid > 0 AND uid = %d)) " .
  'AND submitted > %d AND nid = %d AND is_draft = false';

  // Fetch all the entries from the database within the submit interval with
  // this username and IP.
  $num_submissions_database = db_result(db_query($query, $user->uid, ip_address(), $user->uid, ($node->dquarks['submit_interval'] != -1) ? (time() - $node->dquarks['submit_interval']) : $node->dquarks['submit_interval'], $node->nid));

  // Double check the submission history from the users machine using cookies.
  $num_submissions_cookie = 0;
  if ($user->uid == 0 && variable_get('dquarks_use_cookies', 0)) {
    $cookie_name = 'dquarks-' . $node->nid;

    if (isset($_COOKIE[$cookie_name]) && is_array($_COOKIE[$cookie_name])) {
      foreach ($_COOKIE[$cookie_name] as $key => $timestamp) {
        if ($node->dquarks['submit_interval'] != -1 && $timestamp <= time() - $node->dquarks['submit_interval']) {
          // Remove the cookie if past the required time interval.
          setcookie($cookie_name . '[' . $key . ']', '', 0);
        }
      }
      // Count the number of submissions recorded in cookies.
      $num_submissions_cookie = count($_COOKIE[$cookie_name]);
    }
    else {
      $num_submissions_cookie = 0;
    }
  }

  if ($num_submissions_database >= $node->dquarks['submit_limit'] || $num_submissions_cookie >= $node->dquarks['submit_limit']) {
    // Limit exceeded.
    return TRUE;
  }

  // Limit not exceeded.
  return FALSE;
}
